// GroupCheckBox.cpp : implementation file
//

#include "stdafx.h"
#include "GroupCheckBox.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define ID_CHECKBOX		0xFFFE
#define	LEFT_OFFSET		12
#define	CHECKBOX_SIZE	16


// Data exchange for this control
void AFXAPI DDX_GroupCheck(CDataExchange* pDX, int nIDC, int& value)
{
	// This works because DDX_Check sends BM_GETCHECK and BM_SETCHECK to CGroupCheckBox
	// CGroupCheckBox then recieves those messages and processes them.

	DDX_Check(pDX, nIDC, value);
}

/////////////////////////////////////////////////////////////////////////////
// CGroupCheckBox

CGroupCheckBox::CGroupCheckBox()
{
	m_nStyle = TCH_ALL;
}

CGroupCheckBox::~CGroupCheckBox()
{
}


BEGIN_MESSAGE_MAP(CGroupCheckBox, CButton)
	//{{AFX_MSG_MAP(CGroupCheckBox)
	ON_WM_SETFOCUS()
	ON_WM_ENABLE()
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_CHECKBOX , OnClicked)
	ON_MESSAGE(BM_GETCHECK, OnGetCheck)
	ON_MESSAGE(BM_SETCHECK, OnSetCheck)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGroupCheckBox message handlers

void CGroupCheckBox::OnClicked() 
{
	GroupboxControls(GetCheck());

	::SendMessage(GetParent()->m_hWnd, WM_COMMAND, MAKEWPARAM(::GetDlgCtrlID(m_hWnd), BN_CLICKED), (LPARAM)m_hWnd);
}

void CGroupCheckBox::GroupboxControls(BOOL bActivate)
{
	// Enable/Disable all controls that fall wholey inside of me.
	// Brute force: check every control in the dialog.

	CWnd*	pParent = GetParent();
	CWnd*	pCtrl = pParent->GetWindow(GW_CHILD);	// Gets first child

	CRect	rCtrl, rTest;
	CRect	rGroup;
	GetWindowRect(&rGroup);

	while(pCtrl)
	{
		// Don't do myself !!		
		if(pCtrl == this)
		{
			pCtrl = pCtrl->GetWindow(GW_HWNDNEXT);
			continue;
		}

		if(m_nStyle == TCH_IGNORE && (short)pCtrl->GetDlgCtrlID() == IDC_STATIC)
		{
			pCtrl = pCtrl->GetWindow(GW_HWNDNEXT);
			continue;
		}

		pCtrl->GetWindowRect(&rCtrl);

		if(rTest.IntersectRect(rGroup, rCtrl))
			pCtrl->EnableWindow(bActivate);

		pCtrl = pCtrl->GetWindow(GW_HWNDNEXT);
	}
}

void CGroupCheckBox::SetStyle(TouchStyle nStyle)
{
	m_nStyle = nStyle;

	GroupboxControls(GetCheck());
}


void CGroupCheckBox::PreSubclassWindow() 
{
	// Make sure that this control has the BS_ICON style set.
	// If not, it behaves very strangely:
	//  It erases itself if the user TABs to controls in the dialog,
	//  unless the user first clicks it.  Very strange!!

	ModifyStyle(0, BS_ICON|WS_TABSTOP|WS_GROUP);

	CString	strTitle;
	GetWindowText(strTitle);

	int nWidth = MoveTitle();

	CRect	r;
	GetWindowRect(&r);
	ScreenToClient(r);

	r.OffsetRect(LEFT_OFFSET, 0);
	r.bottom = r.top + CHECKBOX_SIZE;
	r.right = r.left + CHECKBOX_SIZE + nWidth;
	
	m_CheckBox.Create(strTitle, WS_CHILD|BS_AUTOCHECKBOX, r, this, ID_CHECKBOX);
	m_CheckBox.SetFont(GetFont(), true);
	m_CheckBox.ShowWindow(SW_SHOW);
}



int CGroupCheckBox::MoveTitle()
{
	// The group box title needs to be erased, but I need to keep the border away from the check box text.
	// I create a string of spaces (' ') that is the same length as the title was plus the size of the checkbox.
	// Plus a little more.

	CString	strOldTitle, strNewTitle;
	GetWindowText(strOldTitle);

	CClientDC dc(this);
	CFont*	  pOldFont = dc.SelectObject(GetFont());

	CSize	czText	= dc.GetTextExtent(strOldTitle);
	int		nRet	= czText.cx;
	int		nTarget = czText.cx + CHECKBOX_SIZE + 5;
	
	while(czText.cx < nTarget)
	{
		strNewTitle.Insert(0, ' ');
		czText = dc.GetTextExtent(strNewTitle);
	}

	dc.SelectObject(pOldFont);

	SetWindowText(strNewTitle);
	return nRet;
}

void CGroupCheckBox::SetCheck(int nCheck)
{
	m_CheckBox.SetCheck(nCheck);
	GroupboxControls(nCheck==0? false:true);
}

int CGroupCheckBox::GetCheck()
{
	return m_CheckBox.GetCheck();
}


void CGroupCheckBox::OnSetFocus(CWnd* pOldWnd) 
{
	CButton::OnSetFocus(pOldWnd);
	
	m_CheckBox.SetFocus();
}

LRESULT CGroupCheckBox::OnGetCheck(WPARAM wParam, LPARAM lParam)
{
	return GetCheck();
}

LRESULT CGroupCheckBox::OnSetCheck(WPARAM wParam, LPARAM lParam)
{
	SetCheck((int)wParam);
	return 0;
}

void CGroupCheckBox::OnEnable(BOOL bEnable) 
{
	CButton::OnEnable(bEnable);
	
	m_CheckBox.EnableWindow(bEnable);
	GroupboxControls((BOOL)bEnable && GetCheck());
}
